@@ -28,13 +28,15 @@
 
 
 typedef struct {
-	char      *place;           /* All strings contain text in utf8 format. */
+	char      *place;           /* All strings contain text in utf8 
+				     * format. */
 	time_t     time;
 	char      *comment;
 	char     **keywords;
 	int        keywords_n;
-	gboolean   utf8_format;     /* TRUE if text is saved in UTF8 format. gthumb for
-				     * GNOME 1.x saved text in locale format, gthumb for
+	gboolean   utf8_format;     /* TRUE if text is saved in UTF8 format. 
+				     * gthumb for GNOME 1.x saved text in 
+				     * locale format, gthumb for
 				     * GNOME 2.x saves in utf8 format. */
 } CommentData;
 
@@ -61,10 +63,12 @@ gboolean       comment_data_is_void                (CommentData *data);
 /* -- */
 
 char *         comments_get_comment_filename       (const char  *source,
-						    gboolean     resolve_symlinks);
+						    gboolean     resolve_symlinks,
+						    gboolean     unescape);
 
 char *         comments_get_comment_dir            (const char  *directory,
-						    gboolean     resolve_symlinks);
+						    gboolean     resolve_symlinks,
+						    gboolean     unescape);
 
 void           comment_copy                        (const char  *src,
 						    const char  *dest);
@@ -97,5 +101,4 @@ char *         comments_get_comment_as_string      (CommentData *data,
 						    char        *sep1,
 						    char        *sep2);
 
-
 #endif /* COMMENTS_H */
