@@ -350,21 +350,21 @@ visit_rc_dir_data_free (VisitRCDirData *rcd)
 }
 
 
-static void visit_dir_async (const gchar *dir,
+static void visit_dir_async (const char     *dir,
 			     VisitRCDirData *rcd);
 
 
 static void
 rc_path_list_done_cb (PathListData *pld, 
-		      gpointer data)
+		      gpointer      data)
 {
 	VisitRCDirData *rcd = data;
-	GList *scan;
-	gchar *rc_file, *real_file;
-	gchar *sub_dir;
+	GList          *scan;
+	char           *rc_file, *real_file;
+	char           *sub_dir;
 
 	if (pld->result != GNOME_VFS_ERROR_EOF) {
-		gchar *path;
+		char *path;
 
 		path = gnome_vfs_uri_to_string (pld->uri,
 						GNOME_VFS_URI_HIDE_NONE);
@@ -526,12 +526,12 @@ rmdir_recursive (const gchar *directory)
 
 
 gboolean
-path_is_file (const gchar *path)
+path_is_file (const char *path)
 {
 	GnomeVFSFileInfo *info;
-	GnomeVFSResult result;
-	gboolean is_file;
-	gchar *escaped;
+	GnomeVFSResult    result;
+	gboolean          is_file;
+	char             *escaped;
 
 	if (! path || ! *path) return FALSE; 
 
@@ -572,7 +572,9 @@ path_is_dir (const char *path)
 	is_dir = FALSE;
 	if (result == GNOME_VFS_OK)
 		is_dir = (info->type == GNOME_VFS_FILE_TYPE_DIRECTORY);
-	
+	else
+		g_warning ("%s: %s\n", path, gnome_vfs_result_to_string (result));
+
 	g_free (escaped);
 	gnome_vfs_file_info_unref (info);
 
@@ -739,13 +741,21 @@ file_copy (const char *from,
 		return FALSE;
 	}
 
-	while ((n = fread (buf, sizeof (char), BUF_SIZE, fin)) != 0) 
+ retry_read:
+	while ((n = fread (buf, sizeof (char), BUF_SIZE, fin)) != 0) {
+	retry_write:
 		if (fwrite (buf, sizeof (char), n, fout) != n) {
+			if (errno == EINTR)
+				goto retry_write;
 			g_free (dest_dir);
 			fclose (fin);
 			fclose (fout);
 			return FALSE;
 		}
+	}
+	
+	if ((n < 0) && (errno == EINTR))
+		goto retry_read;
 
 	g_free (dest_dir);
 	fclose (fin);
@@ -934,8 +944,8 @@ checksum_simple (const gchar *path)
 
 /* like g_basename but does not warns about NULL and does not
  * alloc a new string. */
-G_CONST_RETURN gchar *
-file_name_from_path (const gchar *file_name)
+G_CONST_RETURN char *
+file_name_from_path (const char *file_name)
 {
 	register gssize base;
         register gssize last_char;
@@ -959,12 +969,12 @@ file_name_from_path (const gchar *file_name)
 }
 
 
-gchar *
-remove_level_from_path (const gchar *path)
+char *
+remove_level_from_path (const char *path)
 {
-	gchar *new_path;
-	const gchar *ptr = path;
-	gint p;
+	int         p;
+	const char *ptr = path;
+	char       *new_path;
 
 	if (! path) 
 		return NULL;
@@ -983,22 +993,24 @@ remove_level_from_path (const gchar *path)
 }
 
 
-gchar *
-remove_extension_from_path (const gchar *path)
+char *
+remove_extension_from_path (const char *path)
 {
-	gchar *new_path;
-	gint len;
-	const gchar *ptr = path;
-	gint p;
+	int         len;
+	int         p;
+	const char *ptr = path;
+	char       *new_path;
 
 	if (! path) 
 		return NULL;
+
 	len = strlen (path);
 	if (len == 1) 
 		return g_strdup (path);
 
 	p = len - 1;
-	while ((ptr[p] != '.') && (p > 0)) p--;
+	while ((ptr[p] != '.') && (p > 0)) 
+		p--;
 	if (p == 0) 
 		p = len;
 	new_path = g_strndup (path, (guint) p);
@@ -1010,7 +1022,7 @@ remove_extension_from_path (const gchar *path)
 gchar * 
 remove_ending_separator (const gchar *path)
 {
-	gint len, copy_len;
+	int len, copy_len;
 
 	if (path == NULL)
 		return NULL;
@@ -1024,38 +1036,43 @@ remove_ending_separator (const gchar *path)
 
 
 gboolean 
-ensure_dir_exists (const gchar *a_path, 
-		   mode_t mode)
+ensure_dir_exists (const char *a_path, 
+		   mode_t      mode)
 {
+	char *path;
+	char *p;
+
 	if (! a_path) return FALSE;
 
-	if (! path_is_dir (a_path)) {
-		gchar *path = g_strdup (a_path);
-		gchar *p = path;
+	if (path_is_dir (a_path))
+		return TRUE;
 
-		while (*p != '\0') {
-			p++;
-			if ((*p == '/') || (*p == '\0')) {
-				gboolean end = TRUE;
-
-				if (*p != '\0') {
-					*p = '\0';
-					end = FALSE;
-				}
-
-			if (! path_is_dir (path)) {
-					if (mkdir (path, mode) < 0) {
-						g_warning ("directory creation failed: %s.", path);
-						g_free (path);
-						return FALSE;
-					}
-				}
-				if (! end) *p = '/';
+	path = g_strdup (a_path);
+	p = path;
+	
+	while (*p != '\0') {
+		p++;
+		if ((*p == '/') || (*p == '\0')) {
+			gboolean end = TRUE;
+			
+			if (*p != '\0') {
+				*p = '\0';
+				end = FALSE;
 			}
+			
+			if (! path_is_dir (path)) {
+				if (mkdir (path, mode) < 0) {
+					g_warning ("directory creation failed: %s.", path);
+					g_free (path);
+					return FALSE;
+				}
+			}
+			if (! end) *p = '/';
 		}
-		g_free (path);
 	}
 
+	g_free (path);
+
 	return TRUE;
 }
 
@@ -1076,8 +1093,8 @@ dir_list_filter_and_sort (GList *dir_list,
 
 		if (! (file_is_hidden (name_only) && ! show_dot_files) 
 		    && (strcmp (name_only, CACHE_DIR) != 0)) {
-			gchar *s;
-			gchar *path = (gchar*) scan->data;
+			char *s;
+			char *path = (char*) scan->data;
 			
 			s = g_strdup (names_only ? name_only : path);
 			filtered = g_list_prepend (filtered, s);
@@ -1397,106 +1414,135 @@ new_uri_from_path (const char *path)
 }
 
 
-/* taken from gnome-vfs-utils.c,
-     Copyright (C) 1999 Free Software Foundation
-     Copyright (C) 2000, 2001 Eazel, Inc.
-*/
-GnomeVFSResult
-resolve_all_symlinks_uri (GnomeVFSURI  *uri,
-			  GnomeVFSURI **result_uri)
+char *
+new_path_from_uri (GnomeVFSURI *uri)
 {
-	GnomeVFSURI      *new_uri, *resolved_uri;
+	char *path;
+	char *unescaped_path;
+
+	path = gnome_vfs_uri_to_string (uri, GNOME_VFS_URI_HIDE_TOPLEVEL_METHOD);
+	unescaped_path = gnome_vfs_unescape_string (path, NULL);
+	g_free (path);
+
+	return unescaped_path;
+}
+
+
+GnomeVFSResult
+resolve_all_symlinks (const char  *text_uri,
+		      char       **resolved_text_uri)
+{
+	GnomeVFSResult    res = GNOME_VFS_OK;
+	char             *my_text_uri;
 	GnomeVFSFileInfo *info;
-	GnomeVFSResult    res;
-	char             *p;
-	int               n_followed_symlinks;
+	const char       *p;
+	int               n_followed_symlinks = 0;
 
-	/* Ref the original uri so we don't lose it */
-	uri = gnome_vfs_uri_ref (uri);
+	*resolved_text_uri = NULL;
 
-	*result_uri = NULL;
+	if (text_uri == NULL) 
+		return res;
 
+	my_text_uri = g_strdup (text_uri);
 	info = gnome_vfs_file_info_new ();
 
-	p = uri->text;
-	n_followed_symlinks = 0;
-	while (*p != 0) {
+	for (p = my_text_uri; (p != NULL) && (*p != 0); ) {
+		char           *new_text_uri;
+		GnomeVFSURI    *new_uri;
+
 		while (*p == GNOME_VFS_URI_PATH_CHR)
 			p++;
 		while (*p != 0 && *p != GNOME_VFS_URI_PATH_CHR)
 			p++;
 
-		new_uri = gnome_vfs_uri_dup (uri);
-		g_free (new_uri->text);
-		new_uri->text = g_strndup (uri->text, p - uri->text);
+		new_text_uri = g_strndup (my_text_uri, p - my_text_uri);
+		new_uri = new_uri_from_path (new_text_uri);
+		g_free (new_text_uri);
 
 		gnome_vfs_file_info_clear (info);
 		res = gnome_vfs_get_file_info_uri (new_uri, info, GNOME_VFS_FILE_INFO_DEFAULT);
+
 		if (res != GNOME_VFS_OK) {
-			gnome_vfs_uri_unref (new_uri);
-			goto out;
-		}
-		if (info->type == GNOME_VFS_FILE_TYPE_SYMBOLIC_LINK &&
-		    info->valid_fields & GNOME_VFS_FILE_INFO_FIELDS_SYMLINK_NAME) {
+			char *old_uri = my_text_uri;
+			my_text_uri = g_build_filename (old_uri, p, NULL);
+			g_free (old_uri);
+			p = NULL;
+
+		} else if (info->type == GNOME_VFS_FILE_TYPE_SYMBOLIC_LINK &&
+			   info->valid_fields & GNOME_VFS_FILE_INFO_FIELDS_SYMLINK_NAME) {
+			GnomeVFSURI *resolved_uri;
+			char        *tmp_resolved_text_uri = NULL;
+
 			n_followed_symlinks++;
 			if (n_followed_symlinks > MAX_SYMLINKS_FOLLOWED) {
 				res = GNOME_VFS_ERROR_TOO_MANY_LINKS;
 				gnome_vfs_uri_unref (new_uri);
 				goto out;
 			}
-			resolved_uri = gnome_vfs_uri_resolve_relative (new_uri,
-								       info->symlink_name);
+
+			resolved_uri = gnome_vfs_uri_resolve_relative (new_uri, info->symlink_name);
+			tmp_resolved_text_uri = new_path_from_uri (resolved_uri);
+			gnome_vfs_uri_unref (resolved_uri);
+
 			if (*p != 0) {
-				gnome_vfs_uri_unref (uri);
-				uri = gnome_vfs_uri_append_string (resolved_uri, p);
-				gnome_vfs_uri_unref (resolved_uri);
+				char *old_uri = my_text_uri;
+				my_text_uri = g_build_filename (tmp_resolved_text_uri, p, NULL);
+				g_free (old_uri);
+				g_free (tmp_resolved_text_uri);
 			} else {
-				gnome_vfs_uri_unref (uri);
-				uri = resolved_uri;
+				g_free (my_text_uri);
+				my_text_uri = tmp_resolved_text_uri;
 			}
 
-			p = uri->text;
-		} 
-		gnome_vfs_uri_unref (new_uri);
+			p = my_text_uri;
+		}
+
+		gnome_vfs_uri_unref (new_uri); 
 	}
 
 	res = GNOME_VFS_OK;
-	*result_uri = gnome_vfs_uri_dup (uri);
+	*resolved_text_uri = my_text_uri;
  out:
 	gnome_vfs_file_info_unref (info);
-	gnome_vfs_uri_unref (uri);
 	return res;
 }
 
 
-/* taken from gnome-vfs-utils.c,
-     Copyright (C) 1999 Free Software Foundation
-     Copyright (C) 2000, 2001 Eazel, Inc.
-*/
-GnomeVFSResult
-resolve_all_symlinks (const char  *text_uri,
-		      char       **resolved_text_uri)
+GnomeVFSFileSize
+get_dest_free_space (const char  *path)
 {
-	GnomeVFSURI    *uri, *resolved_uri;
-	GnomeVFSResult  res;
-
-	*resolved_text_uri = NULL;
-
-	uri = gnome_vfs_uri_new (text_uri);
-	if (uri == NULL || uri->text == NULL) {
-		if (uri != NULL)
-			gnome_vfs_uri_unref (uri);
-		return GNOME_VFS_ERROR_NOT_SUPPORTED;
-	}
-
-	res = resolve_all_symlinks_uri (uri, &resolved_uri);
-
-	if (res == GNOME_VFS_OK) {
-		*resolved_text_uri = gnome_vfs_uri_to_string (resolved_uri, GNOME_VFS_URI_HIDE_TOPLEVEL_METHOD);
-		gnome_vfs_uri_unref (resolved_uri);
-	}
+	GnomeVFSURI      *uri;
+	GnomeVFSResult    result;
+	GnomeVFSFileSize  ret_val;
 
+	uri = new_uri_from_path (path);
+	result = gnome_vfs_get_volume_free_space (uri, &ret_val);
 	gnome_vfs_uri_unref (uri);
 
-	return res;
+	if (result != GNOME_VFS_OK)
+		return (GnomeVFSFileSize) 0;
+	else
+		return ret_val;
+}
+
+
+gboolean
+is_mime_type_writable (const char *mime_type)
+{
+	GSList *list, *scan;
+
+	list = gdk_pixbuf_get_formats();
+	for (scan = list; scan; scan = scan->next) {
+		GdkPixbufFormat *format = scan->data;
+		char **mime_types;
+		int i;
+		mime_types = gdk_pixbuf_format_get_mime_types (format);
+		for (i = 0; mime_types[i] != NULL; i++) 
+			if (strcmp (mime_type, mime_types[i]) == 0)
+				return gdk_pixbuf_format_is_writable (format);
+		g_strfreev (mime_types);
+	}
+	g_slist_free (list);
+	
+	return FALSE;
 }
